package com.diagbot.rule;

import com.diagbot.dto.BillMsg;
import com.diagbot.dto.MatchDTO;
import com.diagbot.dto.PacsOtherDTO;
import com.diagbot.dto.RuleBaseDTO;
import com.diagbot.dto.RuleExtDTO;
import com.diagbot.dto.RuleSimpleDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.BaseTypeEnum;
import com.diagbot.enums.LexiconEnum;
import com.diagbot.enums.TypeEnum;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.MsgNewUtil;
import com.diagbot.util.MsgUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @description: 辅检规则
 * @author: kwz
 * @time: 2020/8/3 14:47
 */
@Component
public class PacsRule {

    @Autowired
    AgeRule ageRule;
    @Autowired
    SexRule sexRule;
    @Autowired
    GroupRule groupRule;
    @Autowired
    CommonRule commonRule;
    @Autowired
    MsgNewUtil msgNewUtil;

    /**
     * 比较辅检名称——开单合理性
     *
     * @param inputList
     * @param ruleBaseDTO
     * @param billMsgList
     * @param conType
     * @param ruleSimpleDTO
     * @param <T>
     */
    public <T> void bill(List<T> inputList, RuleBaseDTO ruleBaseDTO, List<BillMsg> billMsgList, String conType,
                         RuleSimpleDTO ruleSimpleDTO) {
        if (ListUtil.isNotEmpty(inputList)) {
            for (T bean : inputList) {
                String uniqueName = (String) CoreUtil.getFieldValue(bean, "uniqueName");
                String name = (String) CoreUtil.getFieldValue(bean, "name");
                if (StringUtils.isNotBlank(uniqueName) && CoreUtil.compareName(ruleBaseDTO, uniqueName)) {
                    ruleSimpleDTO.setContent(name);
                    ruleSimpleDTO.setConType(conType);
                    BillMsg billMsg = msgNewUtil.getCommonBillMsg(ruleSimpleDTO);
                    CoreUtil.addBeanToList(billMsgList, billMsg);
                }
            }
        }
    }

    /**
     * 比较辅检——危急值
     *
     * @param contentList
     * @param criticalList
     */
    public void critical(List<String> contentList, List<BillMsg> criticalList) {
        for (String s : contentList) {
            BillMsg commonBillMsg = MsgUtil.getCommonCriticalMsg(s, TypeEnum.pacs.getName());
            criticalList.add(commonBillMsg);
        }
    }

    /**
     * 比较辅检结果——其他提醒
     *
     * @param ruleBaseDTOList
     * @param suggestMsg
     * @param otherList
     * @param wordCrfDTO
     * @param ruleExtDTO
     */
    public void otherPacs(List<RuleBaseDTO> ruleBaseDTOList, String suggestMsg, List<BillMsg> otherList,
                          WordCrfDTO wordCrfDTO, RuleExtDTO ruleExtDTO) {
        if (ListUtil.isEmpty(ruleBaseDTOList)) {
            return;
        }
        PacsOtherDTO pacsOtherDTO = new PacsOtherDTO();
        int i = 0;
        out:
        for (RuleBaseDTO ruleBaseDTO : ruleBaseDTOList) {
            if (LexiconEnum.getEnum(ruleBaseDTO.getBaseLibType()) != null) { // 避免空指针
                boolean flag = false;
                switch (LexiconEnum.getEnum(ruleBaseDTO.getBaseLibType())) {
                    case Age: // 年龄
                        flag = ageRule.getFlag(wordCrfDTO, ruleBaseDTO);
                        if (flag) {
                            i++;
                        } else {
                            break out;// 直接跳出for循环
                        }
                        break;
                    case Gender: // 性别
                        flag = sexRule.getFlag(wordCrfDTO, ruleBaseDTO);
                        if (flag) {
                            i++;
                        } else {
                            break out;// 直接跳出for循环
                        }
                        break;
                    case Group: // 人群
                        Map map = groupRule.rule(wordCrfDTO, ruleBaseDTO);
                        if (CoreUtil.getMapFlag(map)) {
                            i++;
                        } else {
                            break out;// 直接跳出for循环
                        }
                        break;
                    case PacsResult: // 辅检结果
                        map = commonRule.compareItem(wordCrfDTO.getPacsLabel().getRes(), ruleBaseDTO);
                        if (ruleBaseDTO.getBaseType().equals(BaseTypeEnum.B3.getKey())) {
                            if (!CoreUtil.getMapFlag(map)) {
                                i++;
                            } else {
                                break out;// 直接跳出for循环
                            }
                        } else {
                            if (CoreUtil.getMapFlag(map)) {
                                i++;
                            } else {
                                break out;// 直接跳出for循环
                            }
                        }
                        break;
                    case Disease: // 诊断
                        map = commonRule.compareItem(wordCrfDTO.getDiag(), ruleBaseDTO);
                        if (CoreUtil.getMapFlag(map)) {
                            i++;
                        } else {
                            break out;// 直接跳出for循环
                        }
                        break;
                    case LisSubName: // 检验结果
                        List<MatchDTO> matchDTOList = CoreUtil.compareLis(ruleBaseDTO, wordCrfDTO.getLis());
                        if (ListUtil.isNotEmpty(matchDTOList)) {
                            i++;
                        } else {
                            break out;// 直接跳出for循环
                        }
                        break;
                    case VitalResult: //体格检查结果
                    case Vital: // 体格检查项目
                        map = CoreUtil.compareVital(ruleBaseDTO, wordCrfDTO.getVitalLabel());
                        if (CoreUtil.getMapFlag(map)) {
                            i++;
                        } else {
                            break out;// 直接跳出for循环
                        }
                        break;
                    case PacsDescribe: // 辅助检查描述
                        matchDTOList = CoreUtil.comparePacsDesc(ruleBaseDTO, wordCrfDTO.getPacsLabel().getPacsNumList());
                        if (ListUtil.isNotEmpty(matchDTOList)) {
                            i++;
                            /**
                             * redis中的基础数据顺序会变动，第一条基础数据不一定是后台首行固定数据，找首行固定数据的依据
                             * 是：基础规则术语类型、名称与主规则一致
                             */
                            if (ruleExtDTO.getLibName().equals(ruleBaseDTO.getBaseLibName())
                                    && ruleExtDTO.getLibType().equals(ruleBaseDTO.getBaseLibType())) {
                                pacsOtherDTO.setMainMsg(matchDTOList.stream().map(r -> r.getContent()).collect(Collectors.toList()));
                            }
                        } else {
                            break out;// 直接跳出for循环
                        }
                        break;
                    default:
                        break;
                }
            }
        }
        // 条件全部符合
        if (i == ruleBaseDTOList.size()) {
            if (ListUtil.isNotEmpty(pacsOtherDTO.getMainMsg())) { // 辅检描述走模板
                for (String s : pacsOtherDTO.getMainMsg()) {
                    BillMsg otherMsg = msgNewUtil.getOtherPacsMsg(s, LexiconEnum.PacsDescribe.getName(), suggestMsg);
                    CoreUtil.addBeanToList(otherList, otherMsg);
                }
            } else { // 其他直接用提示语
                BillMsg otherMsg = MsgUtil.getCommonOtherMsg(TypeEnum.pacs.getName(), suggestMsg, suggestMsg, null, null);
                CoreUtil.addBeanToList(otherList, otherMsg);
            }
        }
    }
}
